#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
"""This module contains an operator to move data from Hive to Samba."""
from __future__ import annotations

from tempfile import NamedTemporaryFile
from typing import TYPE_CHECKING, Sequence

from airflow.models import BaseOperator
from airflow.providers.apache.hive.hooks.hive import HiveServer2Hook
from airflow.providers.samba.hooks.samba import SambaHook
from airflow.utils.operator_helpers import context_to_airflow_vars

if TYPE_CHECKING:
    from airflow.utils.context import Context


class HiveToSambaOperator(BaseOperator):
    """
    Executes hql code in a specific Hive database and loads the
    results of the query as a csv to a Samba location.

    :param hql: the hql to be exported. (templated)
    :param destination_filepath: the file path to where the file will be pushed onto samba
    :param samba_conn_id: reference to the samba destination
    :param hiveserver2_conn_id: Reference to the
        :ref: `Hive Server2 thrift service connection id <howto/connection:hiveserver2>`.
    """

    template_fields: Sequence[str] = ('hql', 'destination_filepath')
    template_ext: Sequence[str] = (
        '.hql',
        '.sql',
    )
    template_fields_renderers = {'hql': 'hql'}

    def __init__(
        self,
        *,
        hql: str,
        destination_filepath: str,
        samba_conn_id: str = 'samba_default',
        hiveserver2_conn_id: str = 'hiveserver2_default',
        **kwargs,
    ) -> None:
        super().__init__(**kwargs)
        self.hiveserver2_conn_id = hiveserver2_conn_id
        self.samba_conn_id = samba_conn_id
        self.destination_filepath = destination_filepath
        self.hql = hql.strip().rstrip(';')

    def execute(self, context: Context):
        with NamedTemporaryFile() as tmp_file:
            self.log.info("Fetching file from Hive")
            hive = HiveServer2Hook(hiveserver2_conn_id=self.hiveserver2_conn_id)
            hive.to_csv(self.hql, csv_filepath=tmp_file.name, hive_conf=context_to_airflow_vars(context))
            self.log.info("Pushing to samba")
            samba = SambaHook(samba_conn_id=self.samba_conn_id)
            samba.push_from_local(self.destination_filepath, tmp_file.name)



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

from typing import TYPE_CHECKING, Any, Sequence

from airflow.sensors.base import BaseSensorOperator

if TYPE_CHECKING:
    from airflow.utils.context import Context


class NamedHivePartitionSensor(BaseSensorOperator):
    """
    Waits for a set of partitions to show up in Hive.

    :param partition_names: List of fully qualified names of the
        partitions to wait for. A fully qualified name is of the
        form ``schema.table/pk1=pv1/pk2=pv2``, for example,
        default.users/ds=2016-01-01. This is passed as is to the metastore
        Thrift client ``get_partitions_by_name`` method. Note that
        you cannot use logical or comparison operators as in
        HivePartitionSensor.
    :param metastore_conn_id: Reference to the
        :ref:`metastore thrift service connection id <howto/connection:hive_metastore>`.
    """

    template_fields: Sequence[str] = ('partition_names',)
    ui_color = '#8d99ae'

    def __init__(
        self,
        *,
        partition_names: list[str],
        metastore_conn_id: str = 'metastore_default',
        poke_interval: int = 60 * 3,
        hook: Any = None,
        **kwargs: Any,
    ):
        super().__init__(poke_interval=poke_interval, **kwargs)

        self.next_index_to_poke = 0
        if isinstance(partition_names, str):
            raise TypeError('partition_names must be an array of strings')

        self.metastore_conn_id = metastore_conn_id
        self.partition_names = partition_names
        self.hook = hook
        if self.hook and metastore_conn_id != 'metastore_default':
            self.log.warning(
                'A hook was passed but a non default metastore_conn_id=%s was used', metastore_conn_id
            )

    @staticmethod
    def parse_partition_name(partition: str) -> tuple[Any, ...]:
        """Get schema, table, and partition info."""
        first_split = partition.split('.', 1)
        if len(first_split) == 1:
            schema = 'default'
            table_partition = max(first_split)  # poor man first
        else:
            schema, table_partition = first_split
        second_split = table_partition.split('/', 1)
        if len(second_split) == 1:
            raise ValueError(f'Could not parse {partition}into table, partition')
        else:
            table, partition = second_split
        return schema, table, partition

    def poke_partition(self, partition: str) -> Any:
        """Check for a named partition."""
        if not self.hook:
            from airflow.providers.apache.hive.hooks.hive import HiveMetastoreHook

            self.hook = HiveMetastoreHook(metastore_conn_id=self.metastore_conn_id)

        schema, table, partition = self.parse_partition_name(partition)

        self.log.info('Poking for %s.%s/%s', schema, table, partition)
        return self.hook.check_for_named_partition(schema, table, partition)

    def poke(self, context: Context) -> bool:

        number_of_partitions = len(self.partition_names)
        poke_index_start = self.next_index_to_poke
        for i in range(number_of_partitions):
            self.next_index_to_poke = (poke_index_start + i) % number_of_partitions
            if not self.poke_partition(self.partition_names[self.next_index_to_poke]):
                return False

        self.next_index_to_poke = 0
        return True



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

# NOTE! THIS FILE IS AUTOMATICALLY GENERATED AND WILL BE
# OVERWRITTEN WHEN PREPARING PACKAGES.
#
# IF YOU WANT TO MODIFY IT, YOU SHOULD MODIFY THE TEMPLATE
# `get_provider_info_TEMPLATE.py.jinja2` IN the `provider_packages` DIRECTORY

def get_provider_info():
    return {
        'package-name': 'apache-airflow-providers-apache-hive',
        'name': 'Apache Hive',
        'description': '`Apache Hive <https://hive.apache.org/>`__\n',
        'versions': [
            '4.0.1',
            '4.0.0',
            '3.1.0',
            '3.0.0',
            '2.3.3',
            '2.3.2',
            '2.3.1',
            '2.3.0',
            '2.2.0',
            '2.1.0',
            '2.0.3',
            '2.0.2',
            '2.0.1',
            '2.0.0',
            '1.0.3',
            '1.0.2',
            '1.0.1',
            '1.0.0',
        ],
        'dependencies': [
            'apache-airflow>=2.2.0',
            'apache-airflow-providers-common-sql>=1.2.0',
            'hmsclient>=0.1.0',
            'pandas>=0.17.1',
            'pyhive[hive]>=0.6.0',
            'sasl>=0.3.1; python_version>="3.9"',
            'thrift>=0.9.2',
        ],
        'integrations': [
            {
                'integration-name': 'Apache Hive',
                'external-doc-url': 'https://hive.apache.org/',
                'how-to-guide': ['/docs/apache-airflow-providers-apache-hive/operators.rst'],
                'logo': '/integration-logos/apache/hive.png',
                'tags': ['apache'],
            }
        ],
        'operators': [
            {
                'integration-name': 'Apache Hive',
                'python-modules': [
                    'airflow.providers.apache.hive.operators.hive',
                    'airflow.providers.apache.hive.operators.hive_stats',
                ],
            }
        ],
        'sensors': [
            {
                'integration-name': 'Apache Hive',
                'python-modules': [
                    'airflow.providers.apache.hive.sensors.hive_partition',
                    'airflow.providers.apache.hive.sensors.metastore_partition',
                    'airflow.providers.apache.hive.sensors.named_hive_partition',
                ],
            }
        ],
        'hooks': [
            {
                'integration-name': 'Apache Hive',
                'python-modules': ['airflow.providers.apache.hive.hooks.hive'],
            }
        ],
        'transfers': [
            {
                'source-integration-name': 'Vertica',
                'target-integration-name': 'Apache Hive',
                'python-module': 'airflow.providers.apache.hive.transfers.vertica_to_hive',
            },
            {
                'source-integration-name': 'Apache Hive',
                'target-integration-name': 'MySQL',
                'python-module': 'airflow.providers.apache.hive.transfers.hive_to_mysql',
            },
            {
                'source-integration-name': 'Apache Hive',
                'target-integration-name': 'Samba',
                'python-module': 'airflow.providers.apache.hive.transfers.hive_to_samba',
            },
            {
                'source-integration-name': 'Amazon Simple Storage Service (S3)',
                'target-integration-name': 'Apache Hive',
                'python-module': 'airflow.providers.apache.hive.transfers.s3_to_hive',
            },
            {
                'source-integration-name': 'MySQL',
                'target-integration-name': 'Apache Hive',
                'python-module': 'airflow.providers.apache.hive.transfers.mysql_to_hive',
            },
            {
                'source-integration-name': 'Microsoft SQL Server (MSSQL)',
                'target-integration-name': 'Apache Hive',
                'python-module': 'airflow.providers.apache.hive.transfers.mssql_to_hive',
            },
        ],
        'connection-types': [
            {
                'hook-class-name': 'airflow.providers.apache.hive.hooks.hive.HiveCliHook',
                'connection-type': 'hive_cli',
            },
            {
                'hook-class-name': 'airflow.providers.apache.hive.hooks.hive.HiveServer2Hook',
                'connection-type': 'hiveserver2',
            },
            {
                'hook-class-name': 'airflow.providers.apache.hive.hooks.hive.HiveMetastoreHook',
                'connection-type': 'hive_metastore',
            },
        ],
    }
